#===============================================================
#         Makefile for generating rmodpfw.exe
#
# Targets:
#         rmodpfw.exe, clean
#
# Usage:
#         To build rmodpfw using gfortran (GNU):
#            $ make -f rmodpfw.mk             ! only first target (default)
#         To clean up:
#            $ make -f rmodpfw.mk clean                       ! second target
#         To build rmodpfw using gfortran (GNU) and clean up:
#            $ make -f rmodpfw.mk rmodpfw.exe clean           ! both targets
#         To run the program:
#            $ chmod a+x rmodpfw.exe
#            $ ./rmodpfw.exe
#
#===============================================================

#--------------------
# Root directory
#--------------------
RP = /home/sveni/SI/oalib
RP = /Your/Path/to/RPRESS/Model 

#------------------------ 
# Source code directories
#-------------------------
r_fw_dir          = $(RP)/rpress/cpm/fw
r_fw2_dir          = $(RP)/rpress/cpm/fw2
r_cpm_dir         = $(RP)/rpress/cpm
r_single_dir      = $(RP)/rpress/cpm/single
r_int_dir         = $(RP)/rpress/int
r_rut_dir         = $(RP)/rpress/rut
r_slatec_dir      = $(RP)/rpress/slatec
linu_dir          = $(RP)/rpress/linu

#-----------------------
# Lists of code to link
#-----------------------
r_fw_list          = FWYSa.o FWPARTa.o FWDISa.o FWYTa.o\
                     fwsubys.o fwysmax.o fwsubytz.o hsolpart.o undox.o
r_fw2_list         = dhivanfw2.o
r_cpm_list         = INMODELa.o PREDHASa.o\
                     azimiln.o dhsub.o ec.o fsubmodelshell.o\
                     harksi.o harksipart.o liqpart.o\
                     u2test.o
r_single_list      = liqsi.o rmodel.o
r_int_list         = dpclin.o
r_rut_list         = eimult.o rhypsmall.o hptrig.o blanktoend.o putintp.o\
                     datan2dq.o cdsqrt12.o\
                     hyprest.o rtext.o cossinc.o
r_slatec_list      = airy2e.o airy4e.o aizbiz.o
linu_list          = d1mach.o

#--------------------
# Common code
#--------------------
OBJECTS = $(r_fw_list) $(r_fw2_list) $(r_cpm_list) $(r_single_list)\
          $(r_int_list) $(r_rut_list) $(r_slatec_list)\
          $(linu_list)

#-----------------------------
# Compilers and compiler flags
#-----------------------------
F90 = /usr/bin/gfortran
FFLAGS  = -fcheck=all -fPIC -ffixed-line-length-132     -fno-align-commons
F_FLAGS =             -fPIC -ffixed-line-length-132 -O3 -fno-align-commons

#------------------
# Compile commands
#------------------
COMPILE.f90 = $(F90) $(FFLAGS) -c
COMPILE_f90 = $(F90) $(F_FLAGS) -c

#---------------
# Link commands
#---------------
LINK.f90 = $(F90) $(FFLAGS) -o
LINK_f90 = $(F90) $(F_FLAGS) -o

#===============================================================

#--------------------
# Link code
#--------------------
rmodpfw.exe: rmodpfw.o modivan.o fwderiv.o $(OBJECTS)
	$(LINK.f90) rmodpfw.exe rmodpfw.o modivan.o fwderiv.o $(OBJECTS)

#===============================================================

#--------------------
# Compile code
#--------------------
rmodpfw.o: $(r_fw_dir)/rmodpfw.f\
                          FWDISa.o FWYTa.o rmodel.o modivan.o fwderiv.o 
	$(COMPILE.f90) $(r_fw_dir)/rmodpfw.f

modivan.o: $(r_fw_dir)/modivan.f\
                          FWYSa.o
	$(COMPILE.f90) $(r_fw_dir)/modivan.f

fwderiv.o: $(r_fw_dir)/fwderiv.f\
                          FWPARTa.o
	$(COMPILE.f90) $(r_fw_dir)/fwderiv.f

#--------------------
# Compile code
#--------------------
FWYSa.o: $(r_fw_dir)/FWYSa.f
	$(COMPILE.f90) $(r_fw_dir)/FWYSa.f

FWPARTa.o: $(r_fw_dir)/FWPARTa.f
	$(COMPILE.f90) $(r_fw_dir)/FWPARTa.f

FWDISa.o: $(r_fw_dir)/FWDISa.f
	$(COMPILE.f90) $(r_fw_dir)/FWDISa.f

FWYTa.o: $(r_fw_dir)/FWYTa.f
	$(COMPILE.f90) $(r_fw_dir)/FWYTa.f

fwsubys.o: $(r_fw_dir)/fwsubys.f
	$(COMPILE.f90) $(r_fw_dir)/fwsubys.f

fwysmax.o: $(r_fw_dir)/fwysmax.f
	$(COMPILE.f90) $(r_fw_dir)/fwysmax.f

fwsubytz.o: $(r_fw_dir)/fwsubytz.f
	$(COMPILE.f90) $(r_fw_dir)/fwsubytz.f

hsolpart.o: $(r_fw_dir)/hsolpart.f
	$(COMPILE.f90) $(r_fw_dir)/hsolpart.f

undox.o: $(r_fw_dir)/undox.f
	$(COMPILE.f90) $(r_fw_dir)/undox.f

dhivanfw2.o: $(r_fw2_dir)/dhivanfw2.f
	$(COMPILE.f90) $(r_fw2_dir)/dhivanfw2.f

INMODELa.o: $(r_cpm_dir)/INMODELa.f
	$(COMPILE.f90) $(r_cpm_dir)/INMODELa.f

PREDHASa.o: $(r_cpm_dir)/PREDHASa.f
	$(COMPILE.f90) $(r_cpm_dir)/PREDHASa.f

azimiln.o: $(r_cpm_dir)/azimiln.f
	$(COMPILE.f90) $(r_cpm_dir)/azimiln.f

dhsub.o: $(r_cpm_dir)/dhsub.f
	$(COMPILE.f90) $(r_cpm_dir)/dhsub.f

ec.o: $(r_cpm_dir)/ec.f
	$(COMPILE.f90) $(r_cpm_dir)/ec.f

fsubmodelshell.o: $(r_cpm_dir)/fsubmodelshell.f
	$(COMPILE.f90) $(r_cpm_dir)/fsubmodelshell.f

harksi.o: $(r_cpm_dir)/harksi.f
	$(COMPILE.f90) $(r_cpm_dir)/harksi.f

harksipart.o: $(r_cpm_dir)/harksipart.f
	$(COMPILE.f90) $(r_cpm_dir)/harksipart.f

liqpart.o: $(r_cpm_dir)/liqpart.f
	$(COMPILE.f90) $(r_cpm_dir)/liqpart.f

u2test.o: $(r_cpm_dir)/u2test.f
	$(COMPILE.f90) $(r_cpm_dir)/u2test.f

liqsi.o: $(r_single_dir)/liqsi.f
	$(COMPILE.f90) $(r_single_dir)/liqsi.f

rmodel.o: $(r_single_dir)/rmodel.f\
                          INMODELa.o
	$(COMPILE.f90) $(r_single_dir)/rmodel.f

dpclin.o: $(r_int_dir)/dpclin.f
	$(COMPILE.f90) $(r_int_dir)/dpclin.f

eimult.o: $(r_rut_dir)/eimult.f
	$(COMPILE.f90) $(r_rut_dir)/eimult.f

rhypsmall.o: $(r_rut_dir)/rhypsmall.f
	$(COMPILE.f90) $(r_rut_dir)/rhypsmall.f

hptrig.o: $(r_rut_dir)/hptrig.f
	$(COMPILE.f90) $(r_rut_dir)/hptrig.f

blanktoend.o: $(r_rut_dir)/blanktoend.f
	$(COMPILE.f90) $(r_rut_dir)/blanktoend.f

putintp.o: $(r_rut_dir)/putintp.f
	$(COMPILE.f90) $(r_rut_dir)/putintp.f

datan2dq.o: $(r_rut_dir)/datan2dq.f
	$(COMPILE.f90) $(r_rut_dir)/datan2dq.f

cdsqrt12.o: $(r_rut_dir)/cdsqrt12.f
	$(COMPILE.f90) $(r_rut_dir)/cdsqrt12.f

hyprest.o: $(r_rut_dir)/hyprest.f
	$(COMPILE.f90) $(r_rut_dir)/hyprest.f

rtext.o: $(r_rut_dir)/rtext.f
	$(COMPILE.f90) $(r_rut_dir)/rtext.f

cossinc.o: $(r_rut_dir)/cossinc.f
	$(COMPILE.f90) $(r_rut_dir)/cossinc.f

airy2e.o: $(r_slatec_dir)/airy2e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy2e.f

airy4e.o: $(r_slatec_dir)/airy4e.f
	$(COMPILE.f90) $(r_slatec_dir)/airy4e.f

aizbiz.o: $(r_slatec_dir)/aizbiz.f\
                        d1mach.o
	$(COMPILE.f90) $(r_slatec_dir)/aizbiz.f

d1mach.o: $(linu_dir)/d1mach.f90
	$(COMPILE.f90) $(linu_dir)/d1mach.f90

#===============================================================

#--------------------
# Auxiliaries
#--------------------
clean:
	rm -rf *.o *.a *~ *.mod

